# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
from pydantic import BaseModel, ConfigDict, Field, StrictStr, ValidationError, field_validator
from typing import Any, List, Optional
from rapidata.api_client.models.i_campaign_filter_campaign_filter import ICampaignFilterCampaignFilter
from rapidata.api_client.models.i_campaign_filter_campaign_session_count_filter import ICampaignFilterCampaignSessionCountFilter
from rapidata.api_client.models.i_campaign_filter_country_filter import ICampaignFilterCountryFilter
from rapidata.api_client.models.i_campaign_filter_demographic_filter import ICampaignFilterDemographicFilter
from rapidata.api_client.models.i_campaign_filter_device_filter import ICampaignFilterDeviceFilter
from rapidata.api_client.models.i_campaign_filter_language_filter import ICampaignFilterLanguageFilter
from rapidata.api_client.models.i_campaign_filter_new_user_filter import ICampaignFilterNewUserFilter
from rapidata.api_client.models.i_campaign_filter_response_count_filter import ICampaignFilterResponseCountFilter
from rapidata.api_client.models.i_campaign_filter_user_action_restriction_filter import ICampaignFilterUserActionRestrictionFilter
from rapidata.api_client.models.i_campaign_filter_user_score_filter import ICampaignFilterUserScoreFilter
from pydantic import StrictStr, Field
from typing import Union, List, Set, Optional, Dict
from typing_extensions import Literal, Self

ICAMPAIGNFILTER_ONE_OF_SCHEMAS = ["ICampaignFilterAndFilter", "ICampaignFilterCampaignFilter", "ICampaignFilterCampaignSessionCountFilter", "ICampaignFilterCountryFilter", "ICampaignFilterDemographicFilter", "ICampaignFilterDeviceFilter", "ICampaignFilterLanguageFilter", "ICampaignFilterNewUserFilter", "ICampaignFilterNotFilter", "ICampaignFilterOrFilter", "ICampaignFilterResponseCountFilter", "ICampaignFilterUserActionRestrictionFilter", "ICampaignFilterUserScoreFilter"]

class ICampaignFilter(BaseModel):
    """
    ICampaignFilter
    """
    # data type: ICampaignFilterAndFilter
    oneof_schema_1_validator: Optional[ICampaignFilterAndFilter] = None
    # data type: ICampaignFilterCampaignFilter
    oneof_schema_2_validator: Optional[ICampaignFilterCampaignFilter] = None
    # data type: ICampaignFilterCampaignSessionCountFilter
    oneof_schema_3_validator: Optional[ICampaignFilterCampaignSessionCountFilter] = None
    # data type: ICampaignFilterCountryFilter
    oneof_schema_4_validator: Optional[ICampaignFilterCountryFilter] = None
    # data type: ICampaignFilterDemographicFilter
    oneof_schema_5_validator: Optional[ICampaignFilterDemographicFilter] = None
    # data type: ICampaignFilterDeviceFilter
    oneof_schema_6_validator: Optional[ICampaignFilterDeviceFilter] = None
    # data type: ICampaignFilterLanguageFilter
    oneof_schema_7_validator: Optional[ICampaignFilterLanguageFilter] = None
    # data type: ICampaignFilterNewUserFilter
    oneof_schema_8_validator: Optional[ICampaignFilterNewUserFilter] = None
    # data type: ICampaignFilterNotFilter
    oneof_schema_9_validator: Optional[ICampaignFilterNotFilter] = None
    # data type: ICampaignFilterOrFilter
    oneof_schema_10_validator: Optional[ICampaignFilterOrFilter] = None
    # data type: ICampaignFilterResponseCountFilter
    oneof_schema_11_validator: Optional[ICampaignFilterResponseCountFilter] = None
    # data type: ICampaignFilterUserActionRestrictionFilter
    oneof_schema_12_validator: Optional[ICampaignFilterUserActionRestrictionFilter] = None
    # data type: ICampaignFilterUserScoreFilter
    oneof_schema_13_validator: Optional[ICampaignFilterUserScoreFilter] = None
    actual_instance: Optional[Union[ICampaignFilterAndFilter, ICampaignFilterCampaignFilter, ICampaignFilterCampaignSessionCountFilter, ICampaignFilterCountryFilter, ICampaignFilterDemographicFilter, ICampaignFilterDeviceFilter, ICampaignFilterLanguageFilter, ICampaignFilterNewUserFilter, ICampaignFilterNotFilter, ICampaignFilterOrFilter, ICampaignFilterResponseCountFilter, ICampaignFilterUserActionRestrictionFilter, ICampaignFilterUserScoreFilter]] = None
    one_of_schemas: Set[str] = { "ICampaignFilterAndFilter", "ICampaignFilterCampaignFilter", "ICampaignFilterCampaignSessionCountFilter", "ICampaignFilterCountryFilter", "ICampaignFilterDemographicFilter", "ICampaignFilterDeviceFilter", "ICampaignFilterLanguageFilter", "ICampaignFilterNewUserFilter", "ICampaignFilterNotFilter", "ICampaignFilterOrFilter", "ICampaignFilterResponseCountFilter", "ICampaignFilterUserActionRestrictionFilter", "ICampaignFilterUserScoreFilter" }

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )


    discriminator_value_class_map: Dict[str, str] = {
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        instance = ICampaignFilter.model_construct()
        error_messages = []
        match = 0
        # validate data type: ICampaignFilterAndFilter
        if not isinstance(v, ICampaignFilterAndFilter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ICampaignFilterAndFilter`")
        else:
            match += 1
        # validate data type: ICampaignFilterCampaignFilter
        if not isinstance(v, ICampaignFilterCampaignFilter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ICampaignFilterCampaignFilter`")
        else:
            match += 1
        # validate data type: ICampaignFilterCampaignSessionCountFilter
        if not isinstance(v, ICampaignFilterCampaignSessionCountFilter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ICampaignFilterCampaignSessionCountFilter`")
        else:
            match += 1
        # validate data type: ICampaignFilterCountryFilter
        if not isinstance(v, ICampaignFilterCountryFilter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ICampaignFilterCountryFilter`")
        else:
            match += 1
        # validate data type: ICampaignFilterDemographicFilter
        if not isinstance(v, ICampaignFilterDemographicFilter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ICampaignFilterDemographicFilter`")
        else:
            match += 1
        # validate data type: ICampaignFilterDeviceFilter
        if not isinstance(v, ICampaignFilterDeviceFilter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ICampaignFilterDeviceFilter`")
        else:
            match += 1
        # validate data type: ICampaignFilterLanguageFilter
        if not isinstance(v, ICampaignFilterLanguageFilter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ICampaignFilterLanguageFilter`")
        else:
            match += 1
        # validate data type: ICampaignFilterNewUserFilter
        if not isinstance(v, ICampaignFilterNewUserFilter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ICampaignFilterNewUserFilter`")
        else:
            match += 1
        # validate data type: ICampaignFilterNotFilter
        if not isinstance(v, ICampaignFilterNotFilter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ICampaignFilterNotFilter`")
        else:
            match += 1
        # validate data type: ICampaignFilterOrFilter
        if not isinstance(v, ICampaignFilterOrFilter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ICampaignFilterOrFilter`")
        else:
            match += 1
        # validate data type: ICampaignFilterResponseCountFilter
        if not isinstance(v, ICampaignFilterResponseCountFilter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ICampaignFilterResponseCountFilter`")
        else:
            match += 1
        # validate data type: ICampaignFilterUserActionRestrictionFilter
        if not isinstance(v, ICampaignFilterUserActionRestrictionFilter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ICampaignFilterUserActionRestrictionFilter`")
        else:
            match += 1
        # validate data type: ICampaignFilterUserScoreFilter
        if not isinstance(v, ICampaignFilterUserScoreFilter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ICampaignFilterUserScoreFilter`")
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when setting `actual_instance` in ICampaignFilter with oneOf schemas: ICampaignFilterAndFilter, ICampaignFilterCampaignFilter, ICampaignFilterCampaignSessionCountFilter, ICampaignFilterCountryFilter, ICampaignFilterDemographicFilter, ICampaignFilterDeviceFilter, ICampaignFilterLanguageFilter, ICampaignFilterNewUserFilter, ICampaignFilterNotFilter, ICampaignFilterOrFilter, ICampaignFilterResponseCountFilter, ICampaignFilterUserActionRestrictionFilter, ICampaignFilterUserScoreFilter. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when setting `actual_instance` in ICampaignFilter with oneOf schemas: ICampaignFilterAndFilter, ICampaignFilterCampaignFilter, ICampaignFilterCampaignSessionCountFilter, ICampaignFilterCountryFilter, ICampaignFilterDemographicFilter, ICampaignFilterDeviceFilter, ICampaignFilterLanguageFilter, ICampaignFilterNewUserFilter, ICampaignFilterNotFilter, ICampaignFilterOrFilter, ICampaignFilterResponseCountFilter, ICampaignFilterUserActionRestrictionFilter, ICampaignFilterUserScoreFilter. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        error_messages = []
        match = 0

        # deserialize data into ICampaignFilterAndFilter
        try:
            instance.actual_instance = ICampaignFilterAndFilter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ICampaignFilterCampaignFilter
        try:
            instance.actual_instance = ICampaignFilterCampaignFilter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ICampaignFilterCampaignSessionCountFilter
        try:
            instance.actual_instance = ICampaignFilterCampaignSessionCountFilter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ICampaignFilterCountryFilter
        try:
            instance.actual_instance = ICampaignFilterCountryFilter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ICampaignFilterDemographicFilter
        try:
            instance.actual_instance = ICampaignFilterDemographicFilter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ICampaignFilterDeviceFilter
        try:
            instance.actual_instance = ICampaignFilterDeviceFilter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ICampaignFilterLanguageFilter
        try:
            instance.actual_instance = ICampaignFilterLanguageFilter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ICampaignFilterNewUserFilter
        try:
            instance.actual_instance = ICampaignFilterNewUserFilter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ICampaignFilterNotFilter
        try:
            instance.actual_instance = ICampaignFilterNotFilter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ICampaignFilterOrFilter
        try:
            instance.actual_instance = ICampaignFilterOrFilter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ICampaignFilterResponseCountFilter
        try:
            instance.actual_instance = ICampaignFilterResponseCountFilter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ICampaignFilterUserActionRestrictionFilter
        try:
            instance.actual_instance = ICampaignFilterUserActionRestrictionFilter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ICampaignFilterUserScoreFilter
        try:
            instance.actual_instance = ICampaignFilterUserScoreFilter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into ICampaignFilter with oneOf schemas: ICampaignFilterAndFilter, ICampaignFilterCampaignFilter, ICampaignFilterCampaignSessionCountFilter, ICampaignFilterCountryFilter, ICampaignFilterDemographicFilter, ICampaignFilterDeviceFilter, ICampaignFilterLanguageFilter, ICampaignFilterNewUserFilter, ICampaignFilterNotFilter, ICampaignFilterOrFilter, ICampaignFilterResponseCountFilter, ICampaignFilterUserActionRestrictionFilter, ICampaignFilterUserScoreFilter. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into ICampaignFilter with oneOf schemas: ICampaignFilterAndFilter, ICampaignFilterCampaignFilter, ICampaignFilterCampaignSessionCountFilter, ICampaignFilterCountryFilter, ICampaignFilterDemographicFilter, ICampaignFilterDeviceFilter, ICampaignFilterLanguageFilter, ICampaignFilterNewUserFilter, ICampaignFilterNotFilter, ICampaignFilterOrFilter, ICampaignFilterResponseCountFilter, ICampaignFilterUserActionRestrictionFilter, ICampaignFilterUserScoreFilter. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], ICampaignFilterAndFilter, ICampaignFilterCampaignFilter, ICampaignFilterCampaignSessionCountFilter, ICampaignFilterCountryFilter, ICampaignFilterDemographicFilter, ICampaignFilterDeviceFilter, ICampaignFilterLanguageFilter, ICampaignFilterNewUserFilter, ICampaignFilterNotFilter, ICampaignFilterOrFilter, ICampaignFilterResponseCountFilter, ICampaignFilterUserActionRestrictionFilter, ICampaignFilterUserScoreFilter]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())

from rapidata.api_client.models.i_campaign_filter_and_filter import ICampaignFilterAndFilter
from rapidata.api_client.models.i_campaign_filter_not_filter import ICampaignFilterNotFilter
from rapidata.api_client.models.i_campaign_filter_or_filter import ICampaignFilterOrFilter
# TODO: Rewrite to not use raise_errors
ICampaignFilter.model_rebuild(raise_errors=False)




# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from datetime import datetime
from pydantic import Field, StrictInt, StrictStr, field_validator
from typing import List, Optional
from typing_extensions import Annotated
from rapidata.api_client.models.flow_get_name_parameter import FlowGetNameParameter
from rapidata.api_client.models.get_completion_time_histogram_endpoint_output import GetCompletionTimeHistogramEndpointOutput
from rapidata.api_client.models.get_flow_by_id_endpoint_output import GetFlowByIdEndpointOutput
from rapidata.api_client.models.get_response_count_histogram_endpoint_output import GetResponseCountHistogramEndpointOutput
from rapidata.api_client.models.get_response_count_timeseries_endpoint_output import GetResponseCountTimeseriesEndpointOutput
from rapidata.api_client.models.query_flows_endpoint_paged_result_of_output import QueryFlowsEndpointPagedResultOfOutput

from rapidata.api_client.api_client import ApiClient, RequestSerialized
from rapidata.api_client.api_response import ApiResponse
from rapidata.api_client.rest import RESTResponseType


class FlowApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def flow_flow_id_delete(
        self,
        flow_id: Annotated[StrictStr, Field(description="The ID of the flow to delete.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Deletes a flow.


        :param flow_id: The ID of the flow to delete. (required)
        :type flow_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._flow_flow_id_delete_serialize(
            flow_id=flow_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': "ValidationProblemDetails",
            '401': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def flow_flow_id_delete_with_http_info(
        self,
        flow_id: Annotated[StrictStr, Field(description="The ID of the flow to delete.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Deletes a flow.


        :param flow_id: The ID of the flow to delete. (required)
        :type flow_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._flow_flow_id_delete_serialize(
            flow_id=flow_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': "ValidationProblemDetails",
            '401': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def flow_flow_id_delete_without_preload_content(
        self,
        flow_id: Annotated[StrictStr, Field(description="The ID of the flow to delete.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Deletes a flow.


        :param flow_id: The ID of the flow to delete. (required)
        :type flow_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._flow_flow_id_delete_serialize(
            flow_id=flow_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': "ValidationProblemDetails",
            '401': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _flow_flow_id_delete_serialize(
        self,
        flow_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if flow_id is not None:
            _path_params['flowId'] = flow_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'OpenIdConnect', 
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/flow/{flowId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def flow_flow_id_get(
        self,
        flow_id: Annotated[StrictStr, Field(description="The ID of the flow to retrieve.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetFlowByIdEndpointOutput:
        """Retrieves a flow by its ID.


        :param flow_id: The ID of the flow to retrieve. (required)
        :type flow_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._flow_flow_id_get_serialize(
            flow_id=flow_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetFlowByIdEndpointOutput",
            '400': "ValidationProblemDetails",
            '401': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def flow_flow_id_get_with_http_info(
        self,
        flow_id: Annotated[StrictStr, Field(description="The ID of the flow to retrieve.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetFlowByIdEndpointOutput]:
        """Retrieves a flow by its ID.


        :param flow_id: The ID of the flow to retrieve. (required)
        :type flow_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._flow_flow_id_get_serialize(
            flow_id=flow_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetFlowByIdEndpointOutput",
            '400': "ValidationProblemDetails",
            '401': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def flow_flow_id_get_without_preload_content(
        self,
        flow_id: Annotated[StrictStr, Field(description="The ID of the flow to retrieve.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Retrieves a flow by its ID.


        :param flow_id: The ID of the flow to retrieve. (required)
        :type flow_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._flow_flow_id_get_serialize(
            flow_id=flow_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetFlowByIdEndpointOutput",
            '400': "ValidationProblemDetails",
            '401': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _flow_flow_id_get_serialize(
        self,
        flow_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if flow_id is not None:
            _path_params['flowId'] = flow_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'OpenIdConnect', 
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/flow/{flowId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def flow_get(
        self,
        page: Annotated[Optional[StrictInt], Field(description="The 1-based page index.")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="The number of items per page.")] = None,
        sort: Annotated[Optional[List[StrictStr]], Field(description="Sort fields. Prefix with - for descending order (e.g. -created_at).")] = None,
        name: Annotated[Optional[FlowGetNameParameter], Field(description="Filter by name.")] = None,
        created_at: Annotated[Optional[FlowGetNameParameter], Field(description="Filter by created_at.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> QueryFlowsEndpointPagedResultOfOutput:
        """Queries flows with filtering and pagination.


        :param page: The 1-based page index.
        :type page: int
        :param page_size: The number of items per page.
        :type page_size: int
        :param sort: Sort fields. Prefix with - for descending order (e.g. -created_at).
        :type sort: List[str]
        :param name: Filter by name.
        :type name: FlowGetNameParameter
        :param created_at: Filter by created_at.
        :type created_at: FlowGetNameParameter
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._flow_get_serialize(
            page=page,
            page_size=page_size,
            sort=sort,
            name=name,
            created_at=created_at,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "QueryFlowsEndpointPagedResultOfOutput",
            '400': "ValidationProblemDetails",
            '401': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def flow_get_with_http_info(
        self,
        page: Annotated[Optional[StrictInt], Field(description="The 1-based page index.")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="The number of items per page.")] = None,
        sort: Annotated[Optional[List[StrictStr]], Field(description="Sort fields. Prefix with - for descending order (e.g. -created_at).")] = None,
        name: Annotated[Optional[FlowGetNameParameter], Field(description="Filter by name.")] = None,
        created_at: Annotated[Optional[FlowGetNameParameter], Field(description="Filter by created_at.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[QueryFlowsEndpointPagedResultOfOutput]:
        """Queries flows with filtering and pagination.


        :param page: The 1-based page index.
        :type page: int
        :param page_size: The number of items per page.
        :type page_size: int
        :param sort: Sort fields. Prefix with - for descending order (e.g. -created_at).
        :type sort: List[str]
        :param name: Filter by name.
        :type name: FlowGetNameParameter
        :param created_at: Filter by created_at.
        :type created_at: FlowGetNameParameter
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._flow_get_serialize(
            page=page,
            page_size=page_size,
            sort=sort,
            name=name,
            created_at=created_at,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "QueryFlowsEndpointPagedResultOfOutput",
            '400': "ValidationProblemDetails",
            '401': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def flow_get_without_preload_content(
        self,
        page: Annotated[Optional[StrictInt], Field(description="The 1-based page index.")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="The number of items per page.")] = None,
        sort: Annotated[Optional[List[StrictStr]], Field(description="Sort fields. Prefix with - for descending order (e.g. -created_at).")] = None,
        name: Annotated[Optional[FlowGetNameParameter], Field(description="Filter by name.")] = None,
        created_at: Annotated[Optional[FlowGetNameParameter], Field(description="Filter by created_at.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Queries flows with filtering and pagination.


        :param page: The 1-based page index.
        :type page: int
        :param page_size: The number of items per page.
        :type page_size: int
        :param sort: Sort fields. Prefix with - for descending order (e.g. -created_at).
        :type sort: List[str]
        :param name: Filter by name.
        :type name: FlowGetNameParameter
        :param created_at: Filter by created_at.
        :type created_at: FlowGetNameParameter
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._flow_get_serialize(
            page=page,
            page_size=page_size,
            sort=sort,
            name=name,
            created_at=created_at,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "QueryFlowsEndpointPagedResultOfOutput",
            '400': "ValidationProblemDetails",
            '401': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _flow_get_serialize(
        self,
        page,
        page_size,
        sort,
        name,
        created_at,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'sort': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if page is not None:
            
            _query_params.append(('page', page))
            
        if page_size is not None:
            
            _query_params.append(('page_size', page_size))
            
        if sort is not None:
            
            _query_params.append(('sort', sort))
            
        if name is not None:
            
            _query_params.append(('name', name))
            
        if created_at is not None:
            
            _query_params.append(('created_at', created_at))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'OpenIdConnect', 
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/flow',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def flow_ranking_flow_id_completion_time_histogram_get(
        self,
        flow_id: Annotated[StrictStr, Field(description="The ID of the flow.")],
        var_from: Annotated[Optional[datetime], Field(description="Optional start of the date range filter (inclusive).")] = None,
        to: Annotated[Optional[datetime], Field(description="Optional end of the date range filter (inclusive).")] = None,
        bucket_count: Annotated[Optional[StrictInt], Field(description="Number of histogram buckets (default 10, max 100).")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetCompletionTimeHistogramEndpointOutput:
        """Returns the distribution of completion durations (in seconds) per completed flow item.


        :param flow_id: The ID of the flow. (required)
        :type flow_id: str
        :param var_from: Optional start of the date range filter (inclusive).
        :type var_from: datetime
        :param to: Optional end of the date range filter (inclusive).
        :type to: datetime
        :param bucket_count: Number of histogram buckets (default 10, max 100).
        :type bucket_count: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._flow_ranking_flow_id_completion_time_histogram_get_serialize(
            flow_id=flow_id,
            var_from=var_from,
            to=to,
            bucket_count=bucket_count,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCompletionTimeHistogramEndpointOutput",
            '400': "ValidationProblemDetails",
            '401': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def flow_ranking_flow_id_completion_time_histogram_get_with_http_info(
        self,
        flow_id: Annotated[StrictStr, Field(description="The ID of the flow.")],
        var_from: Annotated[Optional[datetime], Field(description="Optional start of the date range filter (inclusive).")] = None,
        to: Annotated[Optional[datetime], Field(description="Optional end of the date range filter (inclusive).")] = None,
        bucket_count: Annotated[Optional[StrictInt], Field(description="Number of histogram buckets (default 10, max 100).")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetCompletionTimeHistogramEndpointOutput]:
        """Returns the distribution of completion durations (in seconds) per completed flow item.


        :param flow_id: The ID of the flow. (required)
        :type flow_id: str
        :param var_from: Optional start of the date range filter (inclusive).
        :type var_from: datetime
        :param to: Optional end of the date range filter (inclusive).
        :type to: datetime
        :param bucket_count: Number of histogram buckets (default 10, max 100).
        :type bucket_count: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._flow_ranking_flow_id_completion_time_histogram_get_serialize(
            flow_id=flow_id,
            var_from=var_from,
            to=to,
            bucket_count=bucket_count,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCompletionTimeHistogramEndpointOutput",
            '400': "ValidationProblemDetails",
            '401': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def flow_ranking_flow_id_completion_time_histogram_get_without_preload_content(
        self,
        flow_id: Annotated[StrictStr, Field(description="The ID of the flow.")],
        var_from: Annotated[Optional[datetime], Field(description="Optional start of the date range filter (inclusive).")] = None,
        to: Annotated[Optional[datetime], Field(description="Optional end of the date range filter (inclusive).")] = None,
        bucket_count: Annotated[Optional[StrictInt], Field(description="Number of histogram buckets (default 10, max 100).")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Returns the distribution of completion durations (in seconds) per completed flow item.


        :param flow_id: The ID of the flow. (required)
        :type flow_id: str
        :param var_from: Optional start of the date range filter (inclusive).
        :type var_from: datetime
        :param to: Optional end of the date range filter (inclusive).
        :type to: datetime
        :param bucket_count: Number of histogram buckets (default 10, max 100).
        :type bucket_count: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._flow_ranking_flow_id_completion_time_histogram_get_serialize(
            flow_id=flow_id,
            var_from=var_from,
            to=to,
            bucket_count=bucket_count,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCompletionTimeHistogramEndpointOutput",
            '400': "ValidationProblemDetails",
            '401': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _flow_ranking_flow_id_completion_time_histogram_get_serialize(
        self,
        flow_id,
        var_from,
        to,
        bucket_count,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if flow_id is not None:
            _path_params['flowId'] = flow_id
        # process the query parameters
        if var_from is not None:
            if isinstance(var_from, datetime):
                _query_params.append(
                    (
                        'from',
                        var_from.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('from', var_from))
            
        if to is not None:
            if isinstance(to, datetime):
                _query_params.append(
                    (
                        'to',
                        to.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('to', to))
            
        if bucket_count is not None:
            
            _query_params.append(('bucketCount', bucket_count))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'OpenIdConnect', 
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/flow/ranking/{flowId}/completion-time-histogram',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def flow_ranking_flow_id_response_count_histogram_get(
        self,
        flow_id: Annotated[StrictStr, Field(description="The ID of the flow.")],
        var_from: Annotated[Optional[datetime], Field(description="Optional start of the date range filter (inclusive).")] = None,
        to: Annotated[Optional[datetime], Field(description="Optional end of the date range filter (inclusive).")] = None,
        bucket_count: Annotated[Optional[StrictInt], Field(description="Number of histogram buckets (default 10, max 100).")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetResponseCountHistogramEndpointOutput:
        """Returns the distribution of total votes per completed flow item.


        :param flow_id: The ID of the flow. (required)
        :type flow_id: str
        :param var_from: Optional start of the date range filter (inclusive).
        :type var_from: datetime
        :param to: Optional end of the date range filter (inclusive).
        :type to: datetime
        :param bucket_count: Number of histogram buckets (default 10, max 100).
        :type bucket_count: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._flow_ranking_flow_id_response_count_histogram_get_serialize(
            flow_id=flow_id,
            var_from=var_from,
            to=to,
            bucket_count=bucket_count,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetResponseCountHistogramEndpointOutput",
            '400': "ValidationProblemDetails",
            '401': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def flow_ranking_flow_id_response_count_histogram_get_with_http_info(
        self,
        flow_id: Annotated[StrictStr, Field(description="The ID of the flow.")],
        var_from: Annotated[Optional[datetime], Field(description="Optional start of the date range filter (inclusive).")] = None,
        to: Annotated[Optional[datetime], Field(description="Optional end of the date range filter (inclusive).")] = None,
        bucket_count: Annotated[Optional[StrictInt], Field(description="Number of histogram buckets (default 10, max 100).")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetResponseCountHistogramEndpointOutput]:
        """Returns the distribution of total votes per completed flow item.


        :param flow_id: The ID of the flow. (required)
        :type flow_id: str
        :param var_from: Optional start of the date range filter (inclusive).
        :type var_from: datetime
        :param to: Optional end of the date range filter (inclusive).
        :type to: datetime
        :param bucket_count: Number of histogram buckets (default 10, max 100).
        :type bucket_count: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._flow_ranking_flow_id_response_count_histogram_get_serialize(
            flow_id=flow_id,
            var_from=var_from,
            to=to,
            bucket_count=bucket_count,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetResponseCountHistogramEndpointOutput",
            '400': "ValidationProblemDetails",
            '401': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def flow_ranking_flow_id_response_count_histogram_get_without_preload_content(
        self,
        flow_id: Annotated[StrictStr, Field(description="The ID of the flow.")],
        var_from: Annotated[Optional[datetime], Field(description="Optional start of the date range filter (inclusive).")] = None,
        to: Annotated[Optional[datetime], Field(description="Optional end of the date range filter (inclusive).")] = None,
        bucket_count: Annotated[Optional[StrictInt], Field(description="Number of histogram buckets (default 10, max 100).")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Returns the distribution of total votes per completed flow item.


        :param flow_id: The ID of the flow. (required)
        :type flow_id: str
        :param var_from: Optional start of the date range filter (inclusive).
        :type var_from: datetime
        :param to: Optional end of the date range filter (inclusive).
        :type to: datetime
        :param bucket_count: Number of histogram buckets (default 10, max 100).
        :type bucket_count: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._flow_ranking_flow_id_response_count_histogram_get_serialize(
            flow_id=flow_id,
            var_from=var_from,
            to=to,
            bucket_count=bucket_count,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetResponseCountHistogramEndpointOutput",
            '400': "ValidationProblemDetails",
            '401': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _flow_ranking_flow_id_response_count_histogram_get_serialize(
        self,
        flow_id,
        var_from,
        to,
        bucket_count,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if flow_id is not None:
            _path_params['flowId'] = flow_id
        # process the query parameters
        if var_from is not None:
            if isinstance(var_from, datetime):
                _query_params.append(
                    (
                        'from',
                        var_from.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('from', var_from))
            
        if to is not None:
            if isinstance(to, datetime):
                _query_params.append(
                    (
                        'to',
                        to.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('to', to))
            
        if bucket_count is not None:
            
            _query_params.append(('bucketCount', bucket_count))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'OpenIdConnect', 
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/flow/ranking/{flowId}/response-count-histogram',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def flow_ranking_flow_id_response_count_timeseries_get(
        self,
        flow_id: Annotated[StrictStr, Field(description="The ID of the flow.")],
        var_from: Annotated[Optional[datetime], Field(description="Optional start of the date range filter (inclusive).")] = None,
        to: Annotated[Optional[datetime], Field(description="Optional end of the date range filter (exclusive).")] = None,
        datapoints: Annotated[Optional[StrictInt], Field(description="Number of time buckets (default 100, max 1000).")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetResponseCountTimeseriesEndpointOutput:
        """Returns average response counts bucketed over time for completed flow items.


        :param flow_id: The ID of the flow. (required)
        :type flow_id: str
        :param var_from: Optional start of the date range filter (inclusive).
        :type var_from: datetime
        :param to: Optional end of the date range filter (exclusive).
        :type to: datetime
        :param datapoints: Number of time buckets (default 100, max 1000).
        :type datapoints: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._flow_ranking_flow_id_response_count_timeseries_get_serialize(
            flow_id=flow_id,
            var_from=var_from,
            to=to,
            datapoints=datapoints,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetResponseCountTimeseriesEndpointOutput",
            '400': "ValidationProblemDetails",
            '401': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def flow_ranking_flow_id_response_count_timeseries_get_with_http_info(
        self,
        flow_id: Annotated[StrictStr, Field(description="The ID of the flow.")],
        var_from: Annotated[Optional[datetime], Field(description="Optional start of the date range filter (inclusive).")] = None,
        to: Annotated[Optional[datetime], Field(description="Optional end of the date range filter (exclusive).")] = None,
        datapoints: Annotated[Optional[StrictInt], Field(description="Number of time buckets (default 100, max 1000).")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetResponseCountTimeseriesEndpointOutput]:
        """Returns average response counts bucketed over time for completed flow items.


        :param flow_id: The ID of the flow. (required)
        :type flow_id: str
        :param var_from: Optional start of the date range filter (inclusive).
        :type var_from: datetime
        :param to: Optional end of the date range filter (exclusive).
        :type to: datetime
        :param datapoints: Number of time buckets (default 100, max 1000).
        :type datapoints: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._flow_ranking_flow_id_response_count_timeseries_get_serialize(
            flow_id=flow_id,
            var_from=var_from,
            to=to,
            datapoints=datapoints,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetResponseCountTimeseriesEndpointOutput",
            '400': "ValidationProblemDetails",
            '401': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def flow_ranking_flow_id_response_count_timeseries_get_without_preload_content(
        self,
        flow_id: Annotated[StrictStr, Field(description="The ID of the flow.")],
        var_from: Annotated[Optional[datetime], Field(description="Optional start of the date range filter (inclusive).")] = None,
        to: Annotated[Optional[datetime], Field(description="Optional end of the date range filter (exclusive).")] = None,
        datapoints: Annotated[Optional[StrictInt], Field(description="Number of time buckets (default 100, max 1000).")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Returns average response counts bucketed over time for completed flow items.


        :param flow_id: The ID of the flow. (required)
        :type flow_id: str
        :param var_from: Optional start of the date range filter (inclusive).
        :type var_from: datetime
        :param to: Optional end of the date range filter (exclusive).
        :type to: datetime
        :param datapoints: Number of time buckets (default 100, max 1000).
        :type datapoints: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._flow_ranking_flow_id_response_count_timeseries_get_serialize(
            flow_id=flow_id,
            var_from=var_from,
            to=to,
            datapoints=datapoints,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetResponseCountTimeseriesEndpointOutput",
            '400': "ValidationProblemDetails",
            '401': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _flow_ranking_flow_id_response_count_timeseries_get_serialize(
        self,
        flow_id,
        var_from,
        to,
        datapoints,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if flow_id is not None:
            _path_params['flowId'] = flow_id
        # process the query parameters
        if var_from is not None:
            if isinstance(var_from, datetime):
                _query_params.append(
                    (
                        'from',
                        var_from.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('from', var_from))
            
        if to is not None:
            if isinstance(to, datetime):
                _query_params.append(
                    (
                        'to',
                        to.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('to', to))
            
        if datapoints is not None:
            
            _query_params.append(('datapoints', datapoints))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2', 
            'OpenIdConnect', 
            'Bearer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/flow/ranking/{flowId}/response-count-timeseries',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

# flake8: noqa
"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import os as _os
import re as _re

# Dynamically discover all model modules in this package
def _discover_models() -> dict[str, str]:
    """Discover all model classes from .py files in this package directory."""
    models = {}
    package_dir = _os.path.dirname(_os.path.abspath(__file__))
    package_name = __name__
    
    # Pattern to match class names (PascalCase)
    class_pattern = _re.compile(r'^class\s+([A-Z][A-Za-z0-9]*)\s*[\(:]', _re.MULTILINE)
    
    for filename in _os.listdir(package_dir):
        if filename.endswith('.py') and not filename.startswith('_'):
            module_name = filename[:-3]  # Remove .py extension
            filepath = _os.path.join(package_dir, filename)
            
            try:
                with open(filepath, 'r', encoding='utf-8') as f:
                    content = f.read()
                    # Find all class definitions
                    for match in class_pattern.finditer(content):
                        class_name = match.group(1)
                        # Skip private classes and base classes
                        if not class_name.startswith('_'):
                            models[class_name] = f"{package_name}.{module_name}"
            except Exception:
                pass  # Skip files that can't be read
    
    return models


# Lazy loading for models - models are imported only when accessed
_MODEL_IMPORTS = _discover_models()

# Cache for imported models
_IMPORTED: dict[str, type] = {}


def __getattr__(name: str):
    """Lazy import handler for models."""
    if name in _IMPORTED:
        return _IMPORTED[name]

    if name in _MODEL_IMPORTS:
        module_path = _MODEL_IMPORTS[name]
        import importlib
        module = importlib.import_module(module_path)
        attr = getattr(module, name)
        _IMPORTED[name] = attr
        return attr

    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")


def __dir__():
    """Return list of available attributes for autocompletion."""
    return list(_MODEL_IMPORTS.keys())


# For type checkers and IDE support
__all__ = list(_MODEL_IMPORTS.keys())



from rapidata.rapidata_client.settings._rapidata_setting import RapidataSetting


class PlayVideoUntilTheEnd(RapidataSetting):
    """
    Allows users to only answer once the video has finished playing.
    The additional time gets added on top of the video duration. Can be negative to allow answers before the video ends.

    Args:
        additional_time (int, optional): Additional time in milliseconds. Defaults to 0.
    """

    def __init__(self, additional_time: int = 0):
        if additional_time < -25000 or additional_time > 25000:
            raise ValueError("The additional time must be between -25000 and 25000.")

        super().__init__(
            key="alert_on_fast_response_add_media_duration", value=additional_time
        )



__version__ = "3.2.13"

from .rapidata_client import (
    RapidataClient,
    RapidataAudience,
    RapidataAudienceManager,
    RapidataOrderManager,
    RapidataOrder,
    RapidataJob,
    RapidataJobDefinition,
    RapidataJobManager,
    ValidationSetManager,
    RapidataValidationSet,
    Box,
    RapidataResults,
    DemographicSelection,
    LabelingSelection,
    EffortSelection,
    RapidataRetrievalMode,
    ValidationSelection,
    ConditionalValidationSelection,
    CappedSelection,
    ShufflingSelection,
    RapidataSettings,
    TranslationBehaviourOptions,
    AlertOnFastResponse,
    TranslationBehaviour,
    FreeTextMinimumCharacters,
    NoShuffle,
    PlayVideoUntilTheEnd,
    MuteVideo,
    CustomSetting,
    AllowNeitherBoth,
    SwapContextInstruction,
    CountryFilter,
    LanguageFilter,
    NotFilter,
    OrFilter,
    AndFilter,
    UserScoreFilter,
    CampaignFilter,
    AgeFilter,
    GenderFilter,
    CustomFilter,
    AgeGroup,
    Gender,
    DeviceFilter,
    DeviceType,
    Datapoint,
    FailedUploadException,
    FailedUpload,
    rapidata_config,
    logger,
    managed_print,
)

from . import types



from enum import Enum
from rapidata.api_client.models.retrieval_mode import RetrievalMode


class RapidataRetrievalMode(Enum):
    """
    Enum for defining retrieval modes for datapoints.
    """

    Shuffled = RetrievalMode.SHUFFLED
    """
    Will shuffle the datapoints randomly for each user. The user will then see the datapoints in that order. This will take into account the "max_iterations" parameter.
    """
    Sequential = RetrievalMode.SEQUENTIAL
    """
    Will show the datapoints in the order they are in the dataset. This will take into account the "max_iterations" parameter.
    """
    Random = RetrievalMode.RANDOM
    """
    Will just randomly feed the datapoints to the annotators. This will NOT take into account the "max_iterations" parameter.
    """



# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Union
from rapidata.api_client.models.feature_flag import FeatureFlag
from typing import Optional, Set
from typing_extensions import Self

class UpdateConfigEndpointInput(BaseModel):
    """
    UpdateConfigEndpointInput
    """ # noqa: E501
    criteria: Optional[StrictStr] = Field(default=None, description="The ranking criteria used to compare items.")
    starting_elo: Optional[StrictInt] = Field(default=None, description="Initial Elo rating for new items.", alias="startingElo")
    k_factor: Optional[StrictInt] = Field(default=None, description="K-factor controlling Elo rating sensitivity.", alias="kFactor")
    scaling_factor: Optional[StrictInt] = Field(default=None, description="Scaling factor for Elo probability calculation.", alias="scalingFactor")
    min_responses: Optional[StrictInt] = Field(default=None, description="Minimum number of responses per comparison.", alias="minResponses")
    max_responses: Optional[StrictInt] = Field(default=None, description="Maximum number of responses per comparison.", alias="maxResponses")
    responses_required: Optional[StrictInt] = Field(default=None, description="Deprecated. Use MaxResponses instead.", alias="responsesRequired")
    feature_flags: Optional[List[FeatureFlag]] = Field(default=None, alias="featureFlags")
    target_response_count: Optional[StrictInt] = Field(default=None, description="Target average response count per completed item. Set to null to disable PID control.", alias="targetResponseCount")
    pid_proportional_gain: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="PID proportional gain.", alias="pidProportionalGain")
    pid_integral_gain: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="PID integral gain.", alias="pidIntegralGain")
    pid_derivative_gain: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="PID derivative gain.", alias="pidDerivativeGain")
    pid_min_sessions_per_minute: Optional[StrictInt] = Field(default=None, description="Minimum sessions per minute the PID can set.", alias="pidMinSessionsPerMinute")
    pid_max_sessions_per_minute: Optional[StrictInt] = Field(default=None, description="Maximum sessions per minute the PID can set.", alias="pidMaxSessionsPerMinute")
    pid_batch_mode: Optional[StrictStr] = Field(default=None, description="How PID output maps to campaign rate. Total: direct rate. PerBatch: multiplied by active batch count. PerBatchTimeWeighted: multiplied by time-weighted batch count.", alias="pidBatchMode")
    __properties: ClassVar[List[str]] = ["criteria", "startingElo", "kFactor", "scalingFactor", "minResponses", "maxResponses", "responsesRequired", "featureFlags", "targetResponseCount", "pidProportionalGain", "pidIntegralGain", "pidDerivativeGain", "pidMinSessionsPerMinute", "pidMaxSessionsPerMinute", "pidBatchMode"]

    @field_validator('pid_batch_mode')
    def pid_batch_mode_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Total', 'PerBatch', 'PerBatchTimeWeighted']):
            raise ValueError("must be one of enum values ('Total', 'PerBatch', 'PerBatchTimeWeighted')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of UpdateConfigEndpointInput from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in feature_flags (list)
        _items = []
        if self.feature_flags:
            for _item_feature_flags in self.feature_flags:
                if _item_feature_flags:
                    _items.append(_item_feature_flags.to_dict())
            _dict['featureFlags'] = _items
        # set to None if criteria (nullable) is None
        # and model_fields_set contains the field
        if self.criteria is None and "criteria" in self.model_fields_set:
            _dict['criteria'] = None

        # set to None if starting_elo (nullable) is None
        # and model_fields_set contains the field
        if self.starting_elo is None and "starting_elo" in self.model_fields_set:
            _dict['startingElo'] = None

        # set to None if k_factor (nullable) is None
        # and model_fields_set contains the field
        if self.k_factor is None and "k_factor" in self.model_fields_set:
            _dict['kFactor'] = None

        # set to None if scaling_factor (nullable) is None
        # and model_fields_set contains the field
        if self.scaling_factor is None and "scaling_factor" in self.model_fields_set:
            _dict['scalingFactor'] = None

        # set to None if min_responses (nullable) is None
        # and model_fields_set contains the field
        if self.min_responses is None and "min_responses" in self.model_fields_set:
            _dict['minResponses'] = None

        # set to None if max_responses (nullable) is None
        # and model_fields_set contains the field
        if self.max_responses is None and "max_responses" in self.model_fields_set:
            _dict['maxResponses'] = None

        # set to None if responses_required (nullable) is None
        # and model_fields_set contains the field
        if self.responses_required is None and "responses_required" in self.model_fields_set:
            _dict['responsesRequired'] = None

        # set to None if target_response_count (nullable) is None
        # and model_fields_set contains the field
        if self.target_response_count is None and "target_response_count" in self.model_fields_set:
            _dict['targetResponseCount'] = None

        # set to None if pid_proportional_gain (nullable) is None
        # and model_fields_set contains the field
        if self.pid_proportional_gain is None and "pid_proportional_gain" in self.model_fields_set:
            _dict['pidProportionalGain'] = None

        # set to None if pid_integral_gain (nullable) is None
        # and model_fields_set contains the field
        if self.pid_integral_gain is None and "pid_integral_gain" in self.model_fields_set:
            _dict['pidIntegralGain'] = None

        # set to None if pid_derivative_gain (nullable) is None
        # and model_fields_set contains the field
        if self.pid_derivative_gain is None and "pid_derivative_gain" in self.model_fields_set:
            _dict['pidDerivativeGain'] = None

        # set to None if pid_min_sessions_per_minute (nullable) is None
        # and model_fields_set contains the field
        if self.pid_min_sessions_per_minute is None and "pid_min_sessions_per_minute" in self.model_fields_set:
            _dict['pidMinSessionsPerMinute'] = None

        # set to None if pid_max_sessions_per_minute (nullable) is None
        # and model_fields_set contains the field
        if self.pid_max_sessions_per_minute is None and "pid_max_sessions_per_minute" in self.model_fields_set:
            _dict['pidMaxSessionsPerMinute'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UpdateConfigEndpointInput from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "criteria": obj.get("criteria"),
            "startingElo": obj.get("startingElo"),
            "kFactor": obj.get("kFactor"),
            "scalingFactor": obj.get("scalingFactor"),
            "minResponses": obj.get("minResponses"),
            "maxResponses": obj.get("maxResponses"),
            "responsesRequired": obj.get("responsesRequired"),
            "featureFlags": [FeatureFlag.from_dict(_item) for _item in obj["featureFlags"]] if obj.get("featureFlags") is not None else None,
            "targetResponseCount": obj.get("targetResponseCount"),
            "pidProportionalGain": obj.get("pidProportionalGain"),
            "pidIntegralGain": obj.get("pidIntegralGain"),
            "pidDerivativeGain": obj.get("pidDerivativeGain"),
            "pidMinSessionsPerMinute": obj.get("pidMinSessionsPerMinute"),
            "pidMaxSessionsPerMinute": obj.get("pidMaxSessionsPerMinute"),
            "pidBatchMode": obj.get("pidBatchMode")
        })
        return _obj





from __future__ import annotations

from typing import TYPE_CHECKING

from rapidata.rapidata_client.datapoints._datapoint import Datapoint
from rapidata.service.openapi_service import OpenAPIService
from rapidata.rapidata_client.datapoints._asset_uploader import AssetUploader
from rapidata.rapidata_client.datapoints._asset_mapper import AssetMapper

if TYPE_CHECKING:
    from rapidata.api_client.models.create_datapoint_result import CreateDatapointResult
    from rapidata.api_client.models.i_asset_input import IAssetInput


class DatapointUploader:
    def __init__(self, openapi_service: OpenAPIService):
        self.openapi_service = openapi_service
        self.asset_uploader = AssetUploader(openapi_service)
        self.asset_mapper = AssetMapper()

    def _upload_and_map_asset(self, asset: str | list[str]) -> IAssetInput:
        """Upload asset(s) and map to IAssetInput."""

        if isinstance(asset, list):
            uploaded_names = [self.asset_uploader.upload_asset(a) for a in asset]
            return self.asset_mapper.create_existing_asset_input(uploaded_names)
        else:
            uploaded_name = self.asset_uploader.upload_asset(asset)
            return self.asset_mapper.create_existing_asset_input(uploaded_name)

    def upload_datapoint(
        self, datapoint: Datapoint, dataset_id: str, index: int
    ) -> CreateDatapointResult:
        from rapidata.api_client.models.create_datapoint_model import (
            CreateDatapointModel,
        )

        if datapoint.data_type == "media":
            uploaded_asset = self._upload_and_map_asset(datapoint.asset)
        else:
            uploaded_asset = self.asset_mapper.create_text_input(datapoint.asset)

        return self.openapi_service.dataset_api.dataset_dataset_id_datapoint_post(
            dataset_id=dataset_id,
            create_datapoint_model=CreateDatapointModel(
                asset=uploaded_asset,
                context=datapoint.context,
                contextAsset=(
                    self.asset_mapper.create_existing_asset_input(
                        self.asset_uploader.upload_asset(datapoint.media_context)
                    )
                    if datapoint.media_context
                    else None
                ),
                transcription=datapoint.sentence,
                sortIndex=index,
                group=datapoint.group,
                privateMetadata=datapoint.private_metadata,
            ),
        )



from .asset_upload_exception import AssetUploadException
from .failed_upload import FailedUpload
from .failed_upload_exception import FailedUploadException
from .rapidata_error import RapidataError

__all__ = ["AssetUploadException", "FailedUpload", "FailedUploadException", "RapidataError"]
